package com.gic.store.service.outer;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.service.StoreBrandApiService;
import com.gic.store.service.StoreBrandService;
import com.gic.store.service.StoreService;
import com.gic.store.utils.ErrorCode;
import com.gic.store.utils.MoveType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @date 2019/6/26
 */
@Service("storeBrandApiService")
@SuppressWarnings("unchecked")
public class StoreBrandApiServiceImpl implements StoreBrandApiService {

    private static final Logger logger = LogManager.getLogger(StoreBrandApiServiceImpl.class);

    @Autowired
    private StoreBrandService storeBrandService;

    @Autowired
    private StoreService storeService;

    @Override
    public ServiceResponse<Integer> saveOrUpdateStoreBrand(StoreBrandDTO storeBrandDTO) {
        Integer storeBrandId;
        int brandCodeCount = storeBrandService.countByStoreBrandCode(storeBrandDTO.getEnterpriseId(), storeBrandDTO.getStoreBrandCode(), storeBrandDTO.getStoreBrandId());
        if (brandCodeCount > 0) {
            return ServiceResponse.failure(ErrorCode.ERR_6.getCode(), "品牌code已存在");
        }
        int storeBrandName = storeBrandService.countByStoreBrandName(storeBrandDTO.getEnterpriseId(), storeBrandDTO.getStoreBrandName(), storeBrandDTO.getStoreBrandId());
        if (storeBrandName > 0) {
            return ServiceResponse.failure(ErrorCode.ERR_6.getCode(), "品牌名称已存在");
        }
        if (storeBrandDTO.getStoreBrandId() == null) {
            // 新增
            storeBrandId = storeBrandService.save(storeBrandDTO);
        } else {
            // 编辑
            storeBrandId = storeBrandDTO.getStoreBrandId();
            Integer updateLine = storeBrandService.update(storeBrandDTO);
            if (updateLine == 0) {
                logger.warn("[storeBrandId ==> {}] 不存在", storeBrandDTO.getStoreBrandId());
                return ServiceResponse.failure(ErrorCode.ERR_2.getCode(), "没找到要编辑的门店品牌");
            }
        }
        return ServiceResponse.success(storeBrandId);
    }

    @Override
    public ServiceResponse<Page<StoreBrandDTO>> listStoreBrand(Integer enterpriseId, String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page storeBrandList = storeBrandService.listStoreBrand(enterpriseId, search, pageNum, pageSize);
        Page<StoreBrandDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeBrandList, StoreBrandDTO.class);
        List<StoreBrandDTO> resultList = page.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0; i < resultList.size(); i++) {
            resultList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<List<StoreBrandDTO>> listAllStoreBrand(Integer enterpriseId, String search, Integer storeSupport, Integer goodsSupport) {
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(enterpriseId, search, storeSupport, goodsSupport);
        List<StoreBrandDTO> brandDTOList = EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList);
        return ServiceResponse.success(brandDTOList);
    }

    @Override
    public ServiceResponse<List<Integer>> deleteByIds(Integer enterpriseId, String ids) {
        List<Integer> notDelList = new ArrayList<>();
        if (StringUtils.isNotBlank(ids)) {
            String[] idArr = ids.split(GlobalInfo.FLAG_COMMA);
            List<Integer> delList = Stream.of(idArr)
                    .map(Integer::parseInt)
                    .filter(id -> {
                        Integer count = storeService.countByBrandId(id);
                        if (count > 0) {
                            notDelList.add(count);
                        }
                        return count == 0;
                    })
                    .collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(delList)) {
                storeBrandService.deleteByIds(enterpriseId, delList);
            }
        }
        return ServiceResponse.success(notDelList);
    }

    @Override
    public ServiceResponse moveStoreBrand(Integer enterpriseId, Integer storeBrandId, Integer moveType) {
        if (MoveType.MOVE_UP.getCode().equals(moveType)) {
            // 上移
            storeBrandService.sortMoveUp(enterpriseId, storeBrandId);
        } else if (MoveType.MOVE_DOWN.getCode().equals(moveType)) {
            // 下移
            storeBrandService.sortMoveDown(enterpriseId, storeBrandId);
        } else if (MoveType.TOPPING.getCode().equals(moveType)) {
            // 置顶
            storeBrandService.sortTopping(enterpriseId, storeBrandId);
        } else if (MoveType.BOTTOM.getCode().equals(moveType)) {
            // 置底
            storeBrandService.sortBottom(enterpriseId, storeBrandId);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse setStoreBrandSort(Integer enterpriseId, Integer storeBrandId, Integer seq) {
        storeBrandService.setStoreBrandSort(enterpriseId, storeBrandId, seq);
        return ServiceResponse.success();
    }

}
